# This function aims to convert units
## 
source("modules/Units/R/units_transform.R")
source("modules/Units/R/transform_units_temperature.R")
source("modules/Units/R/transform_units_precipitation.R")
source("modules/Units/R/transform_units_pressure.R")
# Requires recipe to include:
# recipe$Analysis$Variables:
# $freq
# $flux (if precipitation)
# $units (optional)
Units <- function(recipe, data) {
  # from recipe read the user defined units
  # from data the original units
  # deaccumulate option for CDS accumulated variables?
  ncores <- recipe$Analysis$ncores
  ## Do we need to convert other than ECVs?
  var_names <- lapply(data, function(x) {x$attrs$Variable$varName})
  freq <- recipe$Analysis$Variable$freq
  if (is.null(recipe$Analysis$Variable$flux)) {
    flux <- FALSE
  } else {
    flux <- recipe$Analysis$Variable$flux
  }
  orig_units <- list()
  for (element in names(var_names)) {
    orig_units[[element]] <- c()
    for (x in var_names[[element]]) {
      orig_units[[element]] <- c(orig_units[[element]], 
                                 data[[element]]$attrs$Variable$metadata[[x]]$units)
    }
  }
  if (is.null(recipe$Analysis$Variables$units)) {
    user_units <- orig_units[[which(!is.null(orig_units))[1]]]
  } else {
    user_units <- recipe$Analysis$Variables$units
  }
  ## TODO: How to handle spaces in multi-var case?
  user_units <- strsplit(recipe$Analysis$Variables$units, ", | |,")[[1]]
  # remove spaces, "**", "*" and "per" from units
  user_units <- tolower(user_units) 
  user_units <- gsub(" ", "", user_units)
  user_units <- gsub("\\**", "", user_units)
  user_units <- gsub("\\*", "", user_units)
  orig_units <- lapply(orig_units, function(x) {
                       x <- tolower(x)
                       x <- gsub(" ", "", x)
                       x <- gsub("\\**", "", x)
                       x <- gsub("\\*", "", x)
                       x <- unlist(ifelse(rep(length(x)==0, length(user_units)), user_units, x)) #when fcst is NULL
                       }) 
  ## TODO:
  ## if "/" appears substitute by -1 in at the end of next unit. How to know?
  identicalValue <- function(x,y) if (identical(x,y)) TRUE else FALSE
  if (Reduce(identicalValue, c(orig_units, user_units))) { 
    # no transformation needed
    res <- data
  } else {
    res <- sapply(1:length(data), function(x) {
                  if (!all(orig_units[x] == user_units)) {
                    result <- units_transform(data[x], 
                                              orig_units = orig_units[[x]],
                                              user_units = user_units, 
                                              var_names[[x]], freq = freq,
                                              flux = flux,
                                              ncores = ncores)
                  } else {
                    result <- data[x]
                  }
                  return(result)
                  }, simplify = TRUE) # instead of lapply to get the named list directly
    info(recipe$Run$logger, "##### UNIT CONVERSION COMPLETE #####")
  }
  return(res)
}

